# Copyright 2017,2021 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _metrics module.
"""


import re
import pytest

from zhmcclient import Client, MetricsContext, HTTPError, NotFound
from zhmcclient_mock import FakedSession
from tests.common.utils import assert_resources


# Object IDs of our faked metrics contexts:
MC1_OID = 'mc1-oid'
MC2_OID = 'mc2-oid'

# Names of our metric groups:
MG1_NAME = 'partition-usage'
MG2_NAME = 'dpm-system-usage-overview'

# Base URI for metrics contexts (Object ID will be added to get URI):
MC_BASE_URI = '/api/services/metrics/context/'


class TestMetricsContext:
    """All tests for the MetricsContext and MetricsContextManager classes."""

    def setup_method(self):
        """
        Setup that is called by pytest before each test method.

        Set up a faked session, and add a faked CPC in DPM mode without any
        child resources.
        """
        # pylint: disable=attribute-defined-outside-init

        self.session = FakedSession('fake-host', 'fake-hmc', '2.13.1', '1.8')
        self.client = Client(self.session)

    def add_metricscontext1(self):
        """Add faked metrics context 1."""

        faked_metricscontext = self.session.hmc.metrics_contexts.add({
            'fake-id': MC1_OID,
            'anticipated-frequency-seconds': 10,
            'metric-groups': [MG1_NAME, MG2_NAME],
        })
        return faked_metricscontext

    def add_metricscontext2(self):
        """Add faked metrics context 2."""

        faked_metricscontext = self.session.hmc.metrics_contexts.add({
            'fake-id': MC2_OID,
            'anticipated-frequency-seconds': 10,
            'metric-groups': [MG1_NAME],
        })
        return faked_metricscontext

    def create_metricscontext1(self):
        """create (non-faked) metrics context 1."""

        mc_props = {
            'anticipated-frequency-seconds': 10,
            'metric-groups': [MG1_NAME, MG2_NAME],
        }
        metricscontext = self.client.metrics_contexts.create(mc_props)
        return metricscontext

    def create_metricscontext2(self):
        """create (non-faked) metrics context 2."""

        mc_props = {
            'anticipated-frequency-seconds': 10,
            'metric-groups': [MG1_NAME],
        }
        metricscontext = self.client.metrics_contexts.create(mc_props)
        return metricscontext

    def test_mcm_initial_attrs(self):
        """Test initial attributes of MetricsContextManager."""

        metricscontext_mgr = self.client.metrics_contexts

        # Verify all public properties of the manager object
        assert metricscontext_mgr.resource_class == MetricsContext
        assert metricscontext_mgr.session == self.session
        assert metricscontext_mgr.parent is None
        assert metricscontext_mgr.client == self.client

    # TODO: Test for MetricsContextManager.__repr__()

    @pytest.mark.parametrize(
        "full_properties_kwargs, prop_names", [
            ({},
             ['anticipated-frequency-seconds', 'metric-groups']),
            (dict(full_properties=False),
             ['anticipated-frequency-seconds', 'metric-groups']),
            (dict(full_properties=True),
             ['anticipated-frequency-seconds', 'metric-groups']),
        ]
    )
    def test_mcm_list_full_properties(
            self, full_properties_kwargs, prop_names):
        """Test MetricsContextManager.list() with full_properties."""

        # Create (non-faked) metrics contexts (list() will only return those)
        metricscontext1 = self.create_metricscontext1()
        metricscontext2 = self.create_metricscontext2()

        exp_metricscontexts = [metricscontext1, metricscontext2]
        metricscontext_mgr = self.client.metrics_contexts

        # Execute the code to be tested
        metricscontexts = metricscontext_mgr.list(**full_properties_kwargs)

        assert_resources(metricscontexts, exp_metricscontexts, prop_names)

    @pytest.mark.parametrize(
        "filter_args, exp_names", [
            ({'object-id': MC1_OID},
             [MG1_NAME]),
            ({'object-id': MC2_OID},
             [MG2_NAME]),
            ({'object-id': [MC1_OID, MC2_OID]},
             [MG1_NAME, MG2_NAME]),
            ({'object-id': [MC1_OID, MC1_OID]},
             [MG1_NAME]),
            ({'object-id': MC1_OID + 'foo'},
             []),
            ({'object-id': [MC1_OID, MC2_OID + 'foo']},
             [MG1_NAME]),
            ({'object-id': [MC2_OID + 'foo', MC1_OID]},
             [MG1_NAME]),
            ({'name': MG1_NAME},
             [MG1_NAME]),
            ({'name': MG2_NAME},
             [MG2_NAME]),
            ({'name': [MG1_NAME, MG2_NAME]},
             [MG1_NAME, MG2_NAME]),
            ({'name': MG1_NAME + 'foo'},
             []),
            ({'name': [MG1_NAME, MG2_NAME + 'foo']},
             [MG1_NAME]),
            ({'name': [MG2_NAME + 'foo', MG1_NAME]},
             [MG1_NAME]),
            ({'name': [MG1_NAME, MG1_NAME]},
             [MG1_NAME]),
            ({'name': '.*part 1'},
             [MG1_NAME]),
            ({'name': 'part 1.*'},
             [MG1_NAME]),
            ({'name': 'part .'},
             [MG1_NAME, MG2_NAME]),
            ({'name': '.art 1'},
             [MG1_NAME]),
            ({'name': '.+'},
             [MG1_NAME, MG2_NAME]),
            ({'name': 'part 1.+'},
             []),
            ({'name': '.+part 1'},
             []),
            ({'name': MG1_NAME,
              'object-id': MC1_OID},
             [MG1_NAME]),
            ({'name': MG1_NAME,
              'object-id': MC1_OID + 'foo'},
             []),
            ({'name': MG1_NAME + 'foo',
              'object-id': MC1_OID},
             []),
            ({'name': MG1_NAME + 'foo',
              'object-id': MC1_OID + 'foo'},
             []),
        ]
    )
    @pytest.mark.skip  # TODO: Test for MetricsContextManager.list() w/ filter
    def test_mcm_list_filter_args(
            self, filter_args, exp_names):
        """Test MetricsContextManager.list() with filter_args."""

        # Create (non-faked) metrics contexts (list() will only return those)
        self.create_metricscontext1()
        self.create_metricscontext2()

        metricscontext_mgr = self.client.metrics_contexts

        # Execute the code to be tested
        # TODO: Remove pylint disable once supported
        # pylint: disable=unexpected-keyword-arg
        metricscontexts = metricscontext_mgr.list(filter_args=filter_args)

        names = [p.properties['name'] for p in metricscontexts]
        assert set(names) == set(exp_names)

    @pytest.mark.parametrize(
        "input_props, exp_prop_names, exp_exc", [
            ({},
             None,
             HTTPError({'http-status': 400, 'reason': 5})),
            ({'description': 'fake description X'},
             None,
             HTTPError({'http-status': 400, 'reason': 5})),
            ({'name': 'fake-part-x'},
             None,
             HTTPError({'http-status': 400, 'reason': 5})),
            ({'name': 'fake-part-x',
              'initial-memory': 1024},
             None,
             HTTPError({'http-status': 400, 'reason': 5})),
            ({'name': 'fake-part-x',
              'initial-memory': 1024,
              'maximum-memory': 1024},
             ['object-uri', 'name', 'initial-memory', 'maximum-memory'],
             None),
            ({'name': 'fake-part-x',
              'initial-memory': 1024,
              'maximum-memory': 1024,
              'description': 'fake description X'},
             ['object-uri', 'name', 'initial-memory', 'maximum-memory',
              'description'],
             None),
        ]
    )
    @pytest.mark.skip  # TODO: Test for MetricsContextManager.create()
    def test_mcm_create(
            self, input_props, exp_prop_names, exp_exc):
        """Test MetricsContextManager.create()."""

        metricscontext_mgr = self.client.metrics_contexts

        if exp_exc is not None:

            with pytest.raises(exp_exc.__class__) as exc_info:

                # Execute the code to be tested
                metricscontext = metricscontext_mgr.create(
                    properties=input_props)

            exc = exc_info.value
            if isinstance(exp_exc, HTTPError):
                assert exc.http_status == exp_exc.http_status
                assert exc.reason == exp_exc.reason

        else:

            # Execute the code to be tested.
            # Note: the MetricsContext object returned by
            # MetricsContext.create() has the input properties plus
            # 'object-uri'.
            metricscontext = metricscontext_mgr.create(properties=input_props)

            # Check the resource for consistency within itself
            assert isinstance(metricscontext, MetricsContext)
            metricscontext_name = metricscontext.name
            exp_metricscontext_name = metricscontext.properties['name']
            assert metricscontext_name == exp_metricscontext_name
            metricscontext_uri = metricscontext.uri
            exp_metricscontext_uri = metricscontext.properties['object-uri']
            assert metricscontext_uri == exp_metricscontext_uri

            # Check the properties against the expected names and values
            for prop_name in exp_prop_names:
                assert prop_name in metricscontext.properties
                if prop_name in input_props:
                    value = metricscontext.properties[prop_name]
                    exp_value = input_props[prop_name]
                    assert value == exp_value

    @pytest.mark.skip  # TODO: Test for MetricsContextManager.__repr__()
    def test_mc_repr(self):
        """Test MetricsContext.__repr__()."""

        # Add a faked metrics context
        faked_metricscontext = self.add_metricscontext1()

        metricscontext_mgr = self.client.metrics_contexts
        metricscontext = metricscontext_mgr.find(
            name=faked_metricscontext.name)

        # Execute the code to be tested
        repr_str = repr(metricscontext)

        repr_str = repr_str.replace('\n', '\\n')
        # We check just the begin of the string:
        assert re.match(
            rf'^{metricscontext.__class__.__name__}\s+at\s+'
            rf'0x{id(metricscontext):08x}\s+\(\\n.*',
            repr_str)

    @pytest.mark.parametrize(
        "initial_status, exp_exc", [
            ('stopped', None),
            ('terminated', HTTPError({'http-status': 409, 'reason': 1})),
            ('starting', HTTPError({'http-status': 409, 'reason': 1})),
            ('active', HTTPError({'http-status': 409, 'reason': 1})),
            ('stopping', HTTPError({'http-status': 409, 'reason': 1})),
            ('degraded', HTTPError({'http-status': 409, 'reason': 1})),
            ('reservation-error',
             HTTPError({'http-status': 409, 'reason': 1})),
            ('paused', HTTPError({'http-status': 409, 'reason': 1})),
        ]
    )
    @pytest.mark.skip  # TODO: Test for MetricsContext.delete()
    def test_mc_delete(self, initial_status, exp_exc):
        """Test MetricsContext.delete()."""

        # Add a faked metrics context
        faked_metricscontext = self.add_metricscontext1()

        # Set the initial status of the faked metrics context
        faked_metricscontext.properties['status'] = initial_status

        metricscontext_mgr = self.client.metrics_contexts

        metricscontext = metricscontext_mgr.find(
            name=faked_metricscontext.name)

        if exp_exc is not None:

            with pytest.raises(exp_exc.__class__) as exc_info:

                # Execute the code to be tested
                metricscontext.delete()

            exc = exc_info.value
            if isinstance(exp_exc, HTTPError):
                assert exc.http_status == exp_exc.http_status
                assert exc.reason == exp_exc.reason

            # Check that the metrics context still exists
            metricscontext_mgr.find(name=faked_metricscontext.name)

        else:

            # Execute the code to be tested.
            metricscontext.delete()

            # Check that the metrics context no longer exists
            with pytest.raises(NotFound) as exc_info:
                metricscontext_mgr.find(name=faked_metricscontext.name)



# Copyright 2023 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
End2end tests for groups.

These tests do not change any existing groups, but create, modify and delete
groups for testing purposes.
"""


import warnings
import pytest
from requests.packages import urllib3

import zhmcclient
# pylint: disable=line-too-long,unused-import
from zhmcclient.testutils import hmc_definition, hmc_session  # noqa: F401, E501
# pylint: enable=line-too-long,unused-import

from .utils import pick_test_resources, runtest_find_list, TEST_PREFIX, \
    skip_warn, skipif_no_group_support

urllib3.disable_warnings()

# Properties in minimalistic PasswordRule objects (e.g. find_by_name())
GROUP_MINIMAL_PROPS = ['object-uri', 'name']

# Properties in PasswordRule objects returned by list() without full props
GROUP_LIST_PROPS = ['object-uri', 'name']

# Properties whose values can change between retrievals of PasswordRule objects
GROUP_VOLATILE_PROPS = []


def test_group_find_list(hmc_session):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test list(), find(), findall().
    """
    client = zhmcclient.Client(hmc_session)
    console = client.consoles.console
    hd = hmc_session.hmc_definition

    skipif_no_group_support(client)

    # Pick the groups to test with
    group_list = console.groups.list()
    if not group_list:
        skip_warn(f"No groups defined on HMC {hd.host}")
    group_list = pick_test_resources(group_list)

    for group in group_list:
        print(f"Testing with group {group.name!r}")
        runtest_find_list(
            hmc_session, console.groups, group.name, 'name',
            'object-uri', GROUP_VOLATILE_PROPS, GROUP_MINIMAL_PROPS,
            GROUP_LIST_PROPS)


def test_group_crud(hmc_session):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test create, read, update and delete a group.
    """
    client = zhmcclient.Client(hmc_session)
    hd = hmc_session.hmc_definition
    console = client.consoles.console

    skipif_no_group_support(client)

    # TODO: Get group issue on T224 HMC resolved.
    if hd.host == '9.114.87.7':
        skip_warn(f"Issues with group support on HMC {hd.host}")

    group_name = TEST_PREFIX + ' test_group_crud group'

    # Ensure a clean starting point for this test
    try:
        group = console.groups.find(name=group_name)
    except zhmcclient.NotFound:
        pass
    else:
        warnings.warn(
            f"Deleting test group from previous run: {group_name!r}",
            UserWarning)
        group.delete()

    # Test creating the group

    group_input_props = {
        'name': group_name,
        'description': 'Test group for zhmcclient end2end tests',
        'match-info': {},
    }
    group_auto_props = {}

    try:

        # The code to be tested
        try:
            group = console.groups.create(group_input_props)
        except zhmcclient.HTTPError as exc:
            if exc.http_status == 403 and exc.reason == 1:
                skip_warn(
                    f"HMC userid {hd.userid!r} is not authorized for task "
                    f"'Grouping' on HMC {hd.host}")
            else:
                raise

        for pn, exp_value in group_input_props.items():
            assert group.properties[pn] == exp_value, \
                f"Unexpected value for property {pn!r}"
        group.pull_full_properties()
        for pn, exp_value in group_input_props.items():
            assert group.properties[pn] == exp_value, \
                f"Unexpected value for property {pn!r}"
        for pn, exp_value in group_auto_props.items():
            assert group.properties[pn] == exp_value, \
                f"Unexpected value for property {pn!r}"

        # Add a member
        cpc = client.cpcs.list()[0]
        group.add_member(cpc.uri)

        # List members
        members = group.list_members()
        assert len(members) == 1
        member = members[0]
        assert member['object-uri'] == cpc.uri
        assert member['name'] == cpc.name

        # Remove a member
        group.remove_member(cpc.uri)

        # Verify that list of members is empty
        members = group.list_members()
        assert len(members) == 0

    finally:

        # Test deleting the group

        # The code to be tested
        group.delete()

        with pytest.raises(zhmcclient.NotFound):
            console.groups.find(name=group_name)



#!/usr/bin/env python
# Copyright 2018,2022 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example that lists storage groups on a CPC in DPM mode.
"""

import sys
import requests.packages.urllib3

import zhmcclient
from zhmcclient.testutils import hmc_definitions

requests.packages.urllib3.disable_warnings()

# Get HMC info from HMC inventory and vault files
hmc_def = hmc_definitions()[0]
nickname = hmc_def.nickname
host = hmc_def.host
userid = hmc_def.userid
password = hmc_def.password
verify_cert = hmc_def.verify_cert

print(__doc__)

print(f"Using HMC {nickname} at {host} with userid {userid} ...")

print("Creating a session with the HMC ...")
try:
    session = zhmcclient.Session(
        host, userid, password, verify_cert=verify_cert)
except zhmcclient.Error as exc:
    print(f"Error: Cannot establish session with HMC {host}: "
          f"{exc.__class__.__name__}: {exc}")
    sys.exit(1)

try:
    client = zhmcclient.Client(session)

    print("Finding a CPC in DPM mode ...")
    cpcs = client.cpcs.list(filter_args={'dpm-enabled': True})
    if not cpcs:
        print(f"Error: HMC at {host} does not manage any CPCs in DPM mode")
        sys.exit(1)
    cpc = cpcs[0]
    print(f"Using CPC {cpc.name}")

    print(f"Listing storage groups of CPC {cpc.name} ...")
    try:
        storage_groups = cpc.list_associated_storage_groups()
    except zhmcclient.Error as exc:
        print(f"Error: Cannot list storage groups of CPC {cpc.name}: "
              f"{exc.__class__.__name__}: {exc}")
        sys.exit(1)

    for sg in storage_groups:

        print(f"Storage group: {sg.name} (type: {sg.get_property('type')}, "
              f"shared: {sg.get_property('shared')}, "
              f"fulfillment: {sg.get_property('fulfillment-state')})")

        try:
            volumes = sg.storage_volumes.list()
        except zhmcclient.HTTPError as exc:
            print("Error: Cannot list storage volumes of storage group "
                  f"{sg.name}: {exc.__class__.__name__}: {exc}")
            sys.exit(1)

        print(f"    Storage Volumes: {len(volumes)}")
        for sv in volumes:
            print(f"    Storage Volume: {sv.name}")

        if sg.get_property('type') == 'fcp':

            try:
                vsrs = sg.virtual_storage_resources.list()
            except zhmcclient.HTTPError as exc:
                print("Error: Cannot list virtual storage resources of storage "
                      f"group {sg.name}: {exc.__class__.__name__}: {exc}")
                sys.exit(1)

            for vsr in vsrs:
                port = vsr.adapter_port
                adapter = port.manager.parent
                print(f"    Virtual Storage Resource: {vsr.name} "
                      f"(devno: {vsr.get_property('device-number')}, "
                      f"adapter.port: {adapter.name}.{port.name}, "
                      f"attached to partition: {vsr.attached_partition.name})")
            else:
                print("    No Virtual Storage Resources")

finally:
    print("Logging off ...")
    session.logoff()



# Copyright 2022 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for testutils._hmc_vault_file module.
"""


import os
import tempfile
import pytest

from zhmcclient.testutils._hmc_vault_file import HMCVaultFile, \
    HMCVaultFileError


# Test cases for HMCVaultFile.__init__()
TESTCASES_HMC_VAULT_FILE_INIT = [
    # Format:
    # * desc: Testcase description
    # * file_data: String with data of the HMC vault file to test
    # * exp_exc_type: Expected exception type, or None for success
    (
        "No HMCs in vault file",
        """
hmc_auth:
        """,
        None
    ),
    (
        "One HMC with all variables",
        """
hmc_auth:
  east1:
    userid: my_userid
    password: my_password
    verify: true
    ca_certs: my_cert.pem
        """,
        None
    ),
    (
        "One HMC with minimal set of variables",
        """
hmc_auth:
  east1:
    userid: my_userid
    password: my_password
        """,
        None
    ),
    (
        "Two HMCs, and global variable",
        """
hmc_auth:
  east1:
    userid: my_userid
    password: my_password
    verify: true
    ca_certs: my_cert.pem
  east2:
    userid: my_userid2
    password: my_password2
    verify: false
var1: value1
        """,
        None
    ),
    (
        "Schema validation error: Required top-level element missing",
        """
xx:
        """,
        HMCVaultFileError
    ),
    (
        "Schema validation error: List instead of dict",
        """
hmc_auth: []
        """,
        HMCVaultFileError
    ),
    (
        "YAML scanner error",
        """
hmc_auth: xx: yy:
        """,
        HMCVaultFileError
    ),
    (
        "YAML parser error",
        """
 hmc_auth: x
xx: x
        """,
        HMCVaultFileError
    ),
]


@pytest.mark.parametrize(
    "desc, file_data, exp_exc_type",
    TESTCASES_HMC_VAULT_FILE_INIT)
def test_HMCVaultFile_init(desc, file_data, exp_exc_type):
    # pylint: disable=unused-argument,invalid-name
    """
    Test function for HMCVaultFile.__init__().
    """

    with tempfile.NamedTemporaryFile(
            mode='w', suffix='.yaml', delete=False) as tf:
        tf.write(file_data)

    try:
        if exp_exc_type:
            with pytest.raises(exp_exc_type) as exc_info:

                # The function to be tested
                HMCVaultFile(tf.name)

            exc = exc_info.value
            assert isinstance(exc, exp_exc_type)

        else:

            # The function to be tested
            _ = HMCVaultFile(tf.name)

    finally:
        os.remove(tf.name)



# Copyright 2017,2021 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _idpool module of the zhmcclient_mock package.
"""


from requests.packages import urllib3
import pytest

from zhmcclient_mock._idpool import IdPool

urllib3.disable_warnings()


def test_init_error_1():
    """
    Test invalid IdPool.__init__() with lowest > highest.
    """

    with pytest.raises(ValueError):
        IdPool(7, 6)


def test_invalid_free_error_1():
    """
    Test invalid IdPool.free() in some variations.
    """

    pool = IdPool(5, 5)

    with pytest.raises(ValueError):
        pool.free(4)  # not in range

    with pytest.raises(ValueError):
        pool.free(5)  # in range but not allocated

    with pytest.raises(ValueError):
        pool.free(6)  # not in range


def test_invalid_free_error_2():
    """
    Test invalid IdPool.free_if_allocated() in some variations.
    """

    pool = IdPool(5, 5)

    pool.free_if_allocated(4)  # not in range (= not allocated)

    pool.free_if_allocated(5)  # in range but not allocated

    pool.free_if_allocated(6)  # not in range (= not allocated)


def _test_exhausting_for_lo_hi(lowest, highest):
    """
    Internal helper function that tests exhausting an IdPool.
    """

    start = lowest
    end = highest + 1

    pool = IdPool(lowest, highest)

    # Exhaust the pool
    id_list = []
    for _ in range(start, end):
        _id = pool.alloc()
        id_list.append(_id)

    # Verify uniqueness of the ID values
    id_set = set(id_list)
    assert len(id_set) == len(id_list)

    # Verify that the pool is exhausted
    with pytest.raises(ValueError):
        pool.alloc()


def _test_free_for_lo_hi(lowest, highest):
    """
    Internal helper function that tests exhausting, freeing, and again
    exhausting an IdPool.
    """

    start = lowest
    end = highest + 1

    pool = IdPool(lowest, highest)

    # Exhaust the pool
    id_list1 = []
    for _ in range(start, end):
        _id = pool.alloc()
        id_list1.append(_id)

    # Return everything to the pool
    for _id in id_list1:
        pool.free(_id)

    # Verify that nothing is used in the pool
    assert len(pool._used) == 0  # pylint: disable=protected-access

    # Exhaust the pool
    id_list2 = []
    for _ in range(start, end):
        _id = pool.alloc()
        id_list2.append(_id)

    # Verify that the same ID values came back as last time
    assert set(id_list1) == set(id_list2)

    # Verify that the pool is exhausted
    with pytest.raises(ValueError):
        pool.alloc()


TESTCASES_LO_HI = [
    (0, 0),
    (0, 1),
    (0, 9),
    (0, 10),
    (0, 11),
    (0, 19),
    (0, 20),
    (0, 21),
    (3, 3),
    (3, 4),
    (3, 9),
    (3, 10),
    (3, 11),
    (3, 12),
    (3, 13),
    (3, 14),
    (9, 9),
    (9, 10),
    (9, 11),
    (9, 18),
    (9, 19),
    (9, 20),
    (10, 10),
    (10, 11),
    (10, 19),
    (10, 20),
    (10, 21),
    (11, 11),
    (11, 12),
    (11, 20),
    (11, 21),
    (11, 22),
]


@pytest.mark.parametrize("lowest, highest", TESTCASES_LO_HI)
def test_lo_hi(lowest, highest):
    """
    Test pool exhaustion, etc. for some pool sizes.

    Knowing that the chunk size is 10, we focus on the sizes and range
    boundaries around that
    """
    _test_exhausting_for_lo_hi(lowest, highest)
    _test_free_for_lo_hi(lowest, highest)



# Copyright 2017,2021 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
End2end tests for storage volumes (on CPCs in DPM mode).

These tests do not change any existing storage volumes, but create, modify and
delete test storage volumes.
"""


import warnings
import pytest
from requests.packages import urllib3

import zhmcclient
# pylint: disable=line-too-long,unused-import
from zhmcclient.testutils import hmc_definition, hmc_session  # noqa: F401, E501
from zhmcclient.testutils import dpm_mode_cpcs  # noqa: F401, E501
# pylint: enable=line-too-long,unused-import

from .utils import skip_warn, pick_test_resources, TEST_PREFIX, \
    skipif_no_storage_mgmt_feature, runtest_find_list, runtest_get_properties

urllib3.disable_warnings()

# Properties in minimalistic StorageVolume objects (e.g. find_by_name())
STOVOL_MINIMAL_PROPS = ['element-uri', 'name']

# Properties in StorageVolume objects returned by list() without full props
STOVOL_LIST_PROPS = ['element-uri', 'name', 'fulfillment-state', 'size',
                     'usage']

# Properties whose values can change between retrievals of StorageVolume objs
STOVOL_VOLATILE_PROPS = []


def test_stovol_find_list(dpm_mode_cpcs):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test list(), find(), findall().
    """
    if not dpm_mode_cpcs:
        pytest.skip("HMC definition does not include any CPCs in DPM mode")

    for cpc in dpm_mode_cpcs:
        assert cpc.dpm_enabled
        skipif_no_storage_mgmt_feature(cpc)

        session = cpc.manager.session
        hd = session.hmc_definition
        client = cpc.manager.client

        api_version = client.query_api_version()
        hmc_version_str = api_version['hmc-version']
        hmc_version = tuple(map(int, hmc_version_str.split('.')))

        # Pick the storage volumes to test with
        grp_vol_tuples = []
        stogrp_list = cpc.list_associated_storage_groups()
        for stogrp in stogrp_list:
            stovol_list = stogrp.storage_volumes.list()
            for stovol in stovol_list:
                grp_vol_tuples.append((stogrp, stovol))
        if not grp_vol_tuples:
            skip_warn("No storage groups with volumes associated to CPC "
                      f"{cpc.name} managed by HMC {hd.host}")
        grp_vol_tuples = pick_test_resources(grp_vol_tuples)

        # Storage volumes were introduced in HMC 2.14.0 but their names were
        # made unique only in 2.14.1.
        unique_name = (hmc_version >= (2, 14, 1))
        if not unique_name:
            print("Tolerating non-unique storage volume names on HMC "
                  f"version {hmc_version}")
        for stogrp, stovol in grp_vol_tuples:
            print(f"Testing on CPC {cpc.name} with storage volume "
                  f"{stovol.name!r} of storage group {stogrp.name!r}")
            runtest_find_list(
                session, stogrp.storage_volumes, stovol.name, 'name', 'size',
                STOVOL_VOLATILE_PROPS, STOVOL_MINIMAL_PROPS, STOVOL_LIST_PROPS,
                unique_name=unique_name)


def test_stovol_property(dpm_mode_cpcs):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test property related methods
    """
    if not dpm_mode_cpcs:
        pytest.skip("HMC definition does not include any CPCs in DPM mode")

    for cpc in dpm_mode_cpcs:
        assert cpc.dpm_enabled
        skipif_no_storage_mgmt_feature(cpc)

        session = cpc.manager.session
        client = cpc.manager.client
        hd = session.hmc_definition

        api_version = client.query_api_version()
        hmc_version_str = api_version['hmc-version']
        hmc_version = tuple(map(int, hmc_version_str.split('.')))

        # Pick the storage volumes to test with
        grp_vol_tuples = []
        stogrp_list = cpc.list_associated_storage_groups()
        for stogrp in stogrp_list:
            stovol_list = stogrp.storage_volumes.list()
            for stovol in stovol_list:
                grp_vol_tuples.append((stogrp, stovol))
        if not grp_vol_tuples:
            skip_warn("No storage groups with volumes associated to CPC "
                      f"{cpc.name} managed by HMC {hd.host}")
        grp_vol_tuples = pick_test_resources(grp_vol_tuples)

        # Storage volumes were introduced in HMC 2.14.0 but their names were
        # made unique only in 2.14.1.
        unique_name = (hmc_version >= (2, 14, 1))
        if not unique_name:
            print("Tolerating non-unique storage volume names on HMC "
                  f"version {hmc_version}")
        for stogrp, stovol in grp_vol_tuples:
            print(f"Testing on CPC {cpc.name} with storage volume "
                  f"{stovol.name!r} of storage group {stogrp.name!r}")

            # Select a property that is not returned by list()
            non_list_prop = 'description'

            runtest_get_properties(stovol.manager, non_list_prop)


def test_stovol_crud(dpm_mode_cpcs):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test create, read, update and delete a storage volume in a storage group.
    """
    if not dpm_mode_cpcs:
        pytest.skip("HMC definition does not include any CPCs in DPM mode")

    for cpc in dpm_mode_cpcs:
        assert cpc.dpm_enabled
        skipif_no_storage_mgmt_feature(cpc)

        print(f"Testing on CPC {cpc.name}")

        console = cpc.manager.client.consoles.console

        stogrp_name = TEST_PREFIX + ' test_stovol_crud stogrp1'
        stovol_name = 'stovol1'
        stovol_name_new = stovol_name + ' new'

        # Ensure clean starting point
        try:
            stogrp = console.storage_groups.find(name=stogrp_name)
        except zhmcclient.NotFound:
            pass
        else:
            warnings.warn(
                "Deleting test storage group from previous run: "
                f"{stogrp_name!r} on CPC {cpc.name}", UserWarning)
            stogrp.delete()

        stogrp = None
        try:

            # Create a storage group for the volume
            stogrp_input_props = {
                'cpc-uri': cpc.uri,
                'name': stogrp_name,
                'description': 'Dummy storage group description.',
                'type': 'fcp',
            }
            stogrp = console.storage_groups.create(stogrp_input_props)

            # Test creating a volume

            stovol_input_props = {
                'name': stovol_name,
                'description': 'Dummy storage volume description.',
                'size': 100,  # MB
            }
            stovol_auto_props = {
                'fulfillment-state': 'pending',
                'usage': 'data',
            }

            # The code to be tested
            stovol = stogrp.storage_volumes.create(stovol_input_props)

            for pn, exp_value in stovol_input_props.items():
                assert pn in stovol.properties, (
                    f"Input property {pn!r} is not in created storage volume:\n"
                    f"{stovol!r}")
                assert stovol.properties[pn] == exp_value, \
                    f"Unexpected value for property {pn!r} of storage " \
                    f"volume:\n{stovol.properties!r}"
            stovol.pull_full_properties()
            for pn, exp_value in stovol_input_props.items():
                assert stovol.properties[pn] == exp_value, \
                    f"Unexpected value for property {pn!r} of storage " \
                    f"volume:\n{stovol.properties!r}"
            for pn, exp_value in stovol_auto_props.items():
                assert pn in stovol.properties, (
                    f"Automatically returned property {pn!r} is not in "
                    f"created storage volume:\n{stovol!r}")
                assert stovol.properties[pn] == exp_value, \
                    f"Unexpected value for property {pn!r} of storage " \
                    f"volume:\n{stovol.properties!r}"

            # Test updating a property of the storage volume

            new_desc = "Updated storage volume description."

            # The code to be tested
            stovol.update_properties(dict(description=new_desc))

            assert stovol.properties['description'] == new_desc
            stovol.pull_full_properties()
            assert stovol.properties['description'] == new_desc

            # Test renaming the storage volume

            # The code to be tested
            stovol.update_properties(dict(name=stovol_name_new))

            assert stovol.properties['name'] == stovol_name_new
            stovol.pull_full_properties()
            assert stovol.properties['name'] == stovol_name_new
            with pytest.raises(zhmcclient.NotFound):
                stogrp.storage_volumes.find(name=stovol_name)

            # Test deleting the storage volume

            # The code to be tested
            stovol.delete()

            with pytest.raises(zhmcclient.NotFound):
                stogrp.storage_volumes.find(name=stovol_name_new)

        finally:
            # Cleanup
            if stogrp:
                stogrp.delete()



# Copyright 2021 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
End2end tests for LDAP server definitions (on CPCs in DPM mode).

These tests do not change any existing LDAP server definitions, but create,
modify and delete test LDAP server definitions.
"""


import warnings
import pytest
from requests.packages import urllib3

import zhmcclient
# pylint: disable=line-too-long,unused-import
from zhmcclient.testutils import hmc_definition, hmc_session  # noqa: F401, E501
# pylint: enable=line-too-long,unused-import

from .utils import skip_warn, pick_test_resources, TEST_PREFIX, \
    runtest_find_list, runtest_get_properties

urllib3.disable_warnings()

# Properties in minimalistic LDAPServerDefinition objects (e.g. find_by_name())
LDAPSRVDEF_MINIMAL_PROPS = ['element-uri', 'name']

# Properties in LDAPServerDefinition objects returned by list() without full
# props
LDAPSRVDEF_LIST_PROPS = ['element-uri', 'name']

# Properties whose values can change between retrievals of LDAPServerDefinition
# objects
LDAPSRVDEF_VOLATILE_PROPS = []


def test_ldapsrvdef_find_list(hmc_session):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test list(), find(), findall().
    """
    client = zhmcclient.Client(hmc_session)
    console = client.consoles.console
    hd = hmc_session.hmc_definition

    api_version = client.query_api_version()
    hmc_version = api_version['hmc-version']
    hmc_version_info = tuple(map(int, hmc_version.split('.')))
    if hmc_version_info < (2, 13, 0):
        skip_warn(f"HMC {hd.host} of version {hmc_version} does not yet "
                  "support LDAP server definitions")

    # Pick the LDAP server definitions to test with
    ldapsrvdef_list = console.ldap_server_definitions.list()
    if not ldapsrvdef_list:
        skip_warn(f"No LDAP server definitions defined on HMC {hd.host}")
    ldapsrvdef_list = pick_test_resources(ldapsrvdef_list)

    for ldapsrvdef in ldapsrvdef_list:
        print(f"Testing with LDAP server definition {ldapsrvdef.name!r}")
        runtest_find_list(
            hmc_session, console.ldap_server_definitions, ldapsrvdef.name,
            'name', 'element-uri', LDAPSRVDEF_VOLATILE_PROPS,
            LDAPSRVDEF_MINIMAL_PROPS, LDAPSRVDEF_LIST_PROPS)


def test_ldapsrvdef_property(hmc_session):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test property related methods
    """
    client = zhmcclient.Client(hmc_session)
    console = client.consoles.console
    hd = hmc_session.hmc_definition

    api_version = client.query_api_version()
    hmc_version = api_version['hmc-version']
    hmc_version_info = tuple(map(int, hmc_version.split('.')))
    if hmc_version_info < (2, 13, 0):
        skip_warn(f"HMC {hd.host} of version {hmc_version} does not yet "
                  "support LDAP server definitions")

    # Pick the LDAP server definitions to test with
    ldapsrvdef_list = console.ldap_server_definitions.list()
    if not ldapsrvdef_list:
        skip_warn(f"No LDAP server definitions defined on HMC {hd.host}")
    ldapsrvdef_list = pick_test_resources(ldapsrvdef_list)

    for ldapsrvdef in ldapsrvdef_list:
        print(f"Testing with LDAP server definition {ldapsrvdef.name!r}")

        # Select a property that is not returned by list()
        non_list_prop = 'description'

        runtest_get_properties(ldapsrvdef.manager, non_list_prop)


def test_ldapsrvdef_crud(hmc_session):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test create, read, update and delete a LDAP server definition.
    """
    client = zhmcclient.Client(hmc_session)
    console = client.consoles.console
    hd = hmc_session.hmc_definition

    api_version = client.query_api_version()
    hmc_version = api_version['hmc-version']
    hmc_version_info = tuple(map(int, hmc_version.split('.')))
    if hmc_version_info < (2, 13, 0):
        skip_warn(f"HMC {hd.host} of version {hmc_version} does not yet "
                  "support LDAP server definitions")

    ldapsrvdef_name = TEST_PREFIX + ' test_ldapsrvdef_crud ldapsrvdef1'
    ldapsrvdef_name_new = ldapsrvdef_name + ' new'

    # Ensure a clean starting point for this test
    try:
        ldapsrvdef = console.ldap_server_definitions.find(
            name=ldapsrvdef_name)
    except zhmcclient.NotFound:
        pass
    else:
        warnings.warn(
            "Deleting test LDAP server definition from previous run: "
            f"{ldapsrvdef_name!r}", UserWarning)
        ldapsrvdef.delete()

    # Test creating the LDAP server definition

    ldapsrvdef_input_props = {
        'name': ldapsrvdef_name,
        'description': 'Test LDAP server def for zhmcclient end2end tests',
        'primary-hostname-ipaddr': '10.11.12.13',
        'location-method': 'pattern',
        'search-distinguished-name': 'user {0}',
    }
    ldapsrvdef_auto_props = {
        'connection-port': None,
        'use-ssl': False,
    }

    # The code to be tested
    try:
        ldapsrvdef = console.ldap_server_definitions.create(
            ldapsrvdef_input_props)
    except zhmcclient.HTTPError as exc:
        if exc.http_status == 403 and exc.reason == 1:
            skip_warn(f"HMC userid {hd.userid!r} is not authorized for task "
                      f"'Manage LDAP Server Definitions' on HMC {hd.host}")
        else:
            raise

    for pn, exp_value in ldapsrvdef_input_props.items():
        assert ldapsrvdef.properties[pn] == exp_value, \
            f"Unexpected value for property {pn!r}"
    ldapsrvdef.pull_full_properties()
    for pn, exp_value in ldapsrvdef_input_props.items():
        assert ldapsrvdef.properties[pn] == exp_value, \
            f"Unexpected value for property {pn!r}"
    for pn, exp_value in ldapsrvdef_auto_props.items():
        assert ldapsrvdef.properties[pn] == exp_value, \
            f"Unexpected value for property {pn!r}"

    # Test updating a property of the LDAP server definition

    new_desc = "Updated LDAP server definition description."

    # The code to be tested
    ldapsrvdef.update_properties(dict(description=new_desc))

    assert ldapsrvdef.properties['description'] == new_desc
    ldapsrvdef.pull_full_properties()
    assert ldapsrvdef.properties['description'] == new_desc

    # Test that LDAP server definitions cannot be renamed

    with pytest.raises(zhmcclient.HTTPError) as exc_info:

        # The code to be tested
        ldapsrvdef.update_properties(dict(name=ldapsrvdef_name_new))

    exc = exc_info.value
    assert exc.http_status == 400
    assert exc.reason == 6
    with pytest.raises(zhmcclient.NotFound):
        console.ldap_server_definitions.find(name=ldapsrvdef_name_new)

    # Test deleting the LDAP server definition

    # The code to be tested
    ldapsrvdef.delete()

    with pytest.raises(zhmcclient.NotFound):
        console.ldap_server_definitions.find(name=ldapsrvdef_name)



# Copyright 2018,2021 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A :term:`virtual storage resource` object represents a storage-related
z/Architecture device that is visible to a partition and that provides access
for that partition to a :term:`storage volume`.

The storage-related z/Architecture devices that are visible to a partition are
different for the two storage architectures: For FCP, the virtualized HBA is
visible as a device, and the storage volumes (LUNs) are not represented as
devices. For FICON, each ECKD volume is visible as a device, but the
virtualized FICON adapter port is not represented as a device.

What the virtual storage resource objects represent, therefore depends on
the storage architecture of the storage volume they are used for:

* For FCP, a virtual storage resource object represents the virtualized HBA
  in the partition that is used to access the LUN. However, each usage of
  the virtual HBA in context of a storage group has its own virtual storage
  resource object.
* For FICON, a virtual storage resource object represents the ECKD volume.

Virtual storage resource objects are instantiated automatically when a storage
group is attached to a partition, and are removed automatically upon
detachment.

The :term:`HBA` resource objects known from DPM mode before introduction of the
"dpm-storage-management" firmware feature are no longer exposed.

Virtual storage resource objects are contained in :term:`storage group`
objects.

Storage groups and storage volumes only can be defined in CPCs that are in
DPM mode and that have the "dpm-storage-management"
:ref:`firmware feature <firmware features>` enabled.
"""


import re
import copy
# from requests.utils import quote

from ._manager import BaseManager
from ._resource import BaseResource
from ._logging import logged_api_call
from ._utils import RC_VIRTUAL_STORAGE_RESOURCE

__all__ = ['VirtualStorageResourceManager', 'VirtualStorageResource']


class VirtualStorageResourceManager(BaseManager):
    """
    Manager providing access to the :term:`virtual storage resources
    <Virtual Storage Resource>` in a particular :term:`storage group`.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.

    Objects of this class are not directly created by the user; they are
    accessible via the following instance variable of a
    :class:`~zhmcclient.StorageGroup` object:

    * :attr:`~zhmcclient.StorageGroup.virtual_storage_resources`

    HMC/SE version requirements:

    * :ref:`firmware feature <firmware features>` "dpm-storage-management"
    """

    def __init__(self, storage_group):
        # This function should not go into the docs.
        # Parameters:
        #   storage_group (:class:`~zhmcclient.StorageGroup`):
        #     Storage group defining the scope for this manager.

        # Resource properties that are supported as filter query parameters.
        # If the support for a resource property changes within the set of HMC
        # versions that support this type of resource, this list must be set up
        # for the version of the HMC this session is connected to.
        query_props = [
            'name',
            'device-number',
            'adapter-port-uri',
            'partition-uri',
        ]

        super().__init__(
            resource_class=VirtualStorageResource,
            class_name=RC_VIRTUAL_STORAGE_RESOURCE,
            session=storage_group.manager.session,
            parent=storage_group,
            base_uri=f'{storage_group.uri}/virtual-storage-resources',
            oid_prop='element-id',
            uri_prop='element-uri',
            name_prop='name',
            query_props=query_props)

    @property
    def storage_group(self):
        """
        :class:`~zhmcclient.StorageGroup`: :term:`Storage group` defining the
        scope for this manager.
        """
        return self._parent

    @logged_api_call
    def list(self, full_properties=False, filter_args=None):
        """
        List the virtual storage resources in this storage group.

        Any resource property may be specified in a filter argument. For
        details about filter arguments, see :ref:`Filtering`.

        The listing of resources is handled in an optimized way:

        * If this manager is enabled for :ref:`auto-updating`, a locally
          maintained resource list is used (which is automatically updated via
          inventory notifications from the HMC) and the provided filter
          arguments are applied.

        * Otherwise, if the filter arguments specify the resource name as a
          single filter argument with a straight match string (i.e. without
          regular expressions), an optimized lookup is performed based on a
          locally maintained name-URI cache.

        * Otherwise, the HMC List operation is performed with the subset of the
          provided filter arguments that can be handled on the HMC side and the
          remaining filter arguments are applied on the client side on the list
          result.

        HMC/SE version requirements:

        * :ref:`firmware feature <firmware features>` "dpm-storage-management"

        Authorization requirements:

        * Object-access permission to this storage group.

        Parameters:

          full_properties (bool):
            Controls that the full set of resource properties for each returned
            storage volume is being retrieved, vs. only the following short
            set: "element-uri", "name", "device-number", "adapter-port-uri",
            and "partition-uri".

          filter_args (dict):
            Filter arguments that narrow the list of returned resources to
            those that match the specified filter arguments. For details, see
            :ref:`Filtering`.

            `None` causes no filtering to happen, i.e. all resources are
            returned.

        Returns:

          : A list of :class:`~zhmcclient.VirtualStorageResource` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        result_prop = 'virtual-storage-resources'
        list_uri = f'{self.storage_group.uri}/virtual-storage-resources'
        return self._list_with_operation(
            list_uri, result_prop, full_properties, filter_args, None)


class VirtualStorageResource(BaseResource):
    """
    Representation of a :term:`virtual storage resource`.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.

    Objects of this class are not directly created by the user; they are
    returned from creation or list functions on their manager object
    (in this case, :class:`~zhmcclient.VirtualStorageResourceManager`).

    HMC/SE version requirements:

    * :ref:`firmware feature <firmware features>` "dpm-storage-management"
    """

    def __init__(self, manager, uri, name=None, properties=None):
        # This function should not go into the docs.
        #   manager (:class:`~zhmcclient.VirtualStorageResourceManager`):
        #     Manager object for this resource object.
        #   uri (string):
        #     Canonical URI path of the resource.
        #   name (string):
        #     Name of the resource.
        #   properties (dict):
        #     Properties to be set for this resource object. May be `None` or
        #     empty.
        assert isinstance(manager, VirtualStorageResourceManager), (
            "VirtualStorageResource init: Expected manager type "
            f"{VirtualStorageResourceManager}, got {type(manager)}")
        super().__init__(
            manager, uri, name, properties)
        self._attached_partition = None
        self._adapter_port = None
        self._storage_volume = None

    @property
    def attached_partition(self):
        """
        :class:`~zhmcclient.Partition`: The partition to which this virtual
        storage resource is attached.

        The returned partition object has only a minimal set of properties set
        ('object-id', 'object-uri', 'class', 'parent').

        Note that a virtual storage resource is always attached to a partition,
        as long as it exists.

        HMC/SE version requirements:

        * :ref:`firmware feature <firmware features>` "dpm-storage-management"

        Authorization requirements:

        * Object-access permission to the storage group owning this
          virtual storage resource.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        if self._attached_partition is None:
            part_mgr = self.manager.storage_group.manager.cpc.partitions
            part = part_mgr.resource_object(self.get_property('partition-uri'))
            self._attached_partition = part
        return self._attached_partition

    @property
    def adapter_port(self):
        """
        :class:`~zhmcclient.Port`: The storage adapter port associated with
        this virtual storage resource, once discovery has determined which
        port to use for this virtual storage resource.

        This applies to both, FCP and FICON/ECKD typed storage groups.

        The returned adapter port object has only a minimal set of properties
        set ('object-id', 'object-uri', 'class', 'parent').

        HMC/SE version requirements:

        * :ref:`firmware feature <firmware features>` "dpm-storage-management"

        Authorization requirements:

        * Object-access permission to the storage group owning this
          virtual storage resource.
        * Object-access permission to the CPC of the storage adapter.
        * Object-access permission to the storage adapter.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        if self._adapter_port is None:
            port_uri = self.get_property('adapter-port-uri')
            assert port_uri is not None
            m = re.match(r'^(/api/adapters/[^/]+)/.*', port_uri)
            adapter_uri = m.group(1)
            adapter_mgr = self.manager.storage_group.cpc.adapters
            filter_args = {'object-uri': adapter_uri}
            adapter = adapter_mgr.find(**filter_args)
            port_mgr = adapter.ports
            port = port_mgr.resource_object(port_uri)
            self._adapter_port = port
        return self._adapter_port

    @logged_api_call
    def update_properties(self, properties):
        """
        Update writeable properties of this virtual storage resource.

        This method serializes with other methods that access or change
        properties on the same Python object.

        HMC/SE version requirements:

        * :ref:`firmware feature <firmware features>` "dpm-storage-management"

        Authorization requirements:

        * Object-access permission to the storage group owning this
          virtual storage resource.
        * Task permission to the "Configure Storage - System Programmer" task.

        Parameters:

          properties (dict): New values for the properties to be updated.
            Properties not to be updated are omitted.
            Allowable properties are the properties with qualifier (w) in
            section 'Data model' in section 'Virtual Storage Resource object'
            in the :term:`HMC API` book.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        # pylint: disable=protected-access
        self.manager.session.post(self.uri, resource=self, body=properties)
        is_rename = self.manager._name_prop in properties
        if is_rename:
            # Delete the old name from the cache
            self.manager._name_uri_cache.delete(self.name)
        self.update_properties_local(copy.deepcopy(properties))
        if is_rename:
            # Add the new name to the cache
            self.manager._name_uri_cache.update(self.name, self.uri)



#!/usr/bin/env python
# Copyright 2021,2022 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example that uses the auto-updating of resource properties on a partition on
a CPC in DPM mode.
"""

import sys
import uuid
import requests.packages.urllib3
from time import sleep

import zhmcclient
from zhmcclient.testutils import hmc_definitions

requests.packages.urllib3.disable_warnings()

# Get HMC info from HMC inventory and vault files
hmc_def = hmc_definitions()[0]
nickname = hmc_def.nickname
host = hmc_def.host
userid = hmc_def.userid
password = hmc_def.password
verify_cert = hmc_def.verify_cert

print(__doc__)

print(f"Using HMC {nickname} at {host} with userid {userid} ...")

print("Creating a session with the HMC ...")
try:
    session = zhmcclient.Session(
        host, userid, password, verify_cert=verify_cert)
except zhmcclient.Error as exc:
    print(f"Error: Cannot establish session with HMC {host}: "
          f"{exc.__class__.__name__}: {exc}")
    sys.exit(1)

try:
    client = zhmcclient.Client(session)

    print("Finding a CPC in DPM mode ...")
    cpcs = client.cpcs.list(filter_args={'dpm-enabled': True})
    if not cpcs:
        print(f"Error: HMC at {host} does not manage any CPCs in DPM mode")
        sys.exit(1)
    cpc = cpcs[0]
    print(f"Using CPC {cpc.name}")

    part_name = f"zhmc_test_{uuid.uuid4()}"
    print(f"Creating partition {part_name} ...")
    part_props = {
        'name': part_name,
        'description': 'Original partition description.',
        'cp-processors': 2,
        'initial-memory': 1024,
        'maximum-memory': 2048,
        'processor-mode': 'shared',
        'boot-device': 'test-operating-system'
    }
    try:
        part = cpc.partitions.create(properties=part_props)
    except zhmcclient.Error as exc:
        print(f"Error: Cannot create partition {part_name} on CPC {cpc.name}: "
              f"{exc.__class__.__name__}: {exc}")
        sys.exit(1)

    try:
        obj_topic = session.object_topic
        print(f"Object notification topic: {obj_topic}")

        # Two different zhmcclient.Partition objects representing the same
        # partition on the HMC
        partition1 = cpc.partitions.find(name=part_name)
        partition2 = cpc.partitions.find(name=part_name)

        # The property that will be updated
        part_prop = 'description'

        print("Enabling auto-update for partition object 1 for partition "
              f"{part_name} (id={id(partition1)})")
        partition1.enable_auto_update()

        print("Not enabling auto-update for partition object 2 for partition "
              f"{part_name} (id={id(partition2)})")

        print(f"Entering loop that displays property '{part_prop}' of "
              "partition objects 1 and 2")
        print("")
        print(f"==> Update property '{part_prop}' of partition '{part_name}' "
              f"on CPC '{cpc.name}' from another session")
        print(f"==> Delete partition '{part_name}' on CPC '{cpc.name}' from "
              "another session")
        print("")
        try:
            while True:
                try:
                    value1 = partition1.prop(part_prop)
                except zhmcclient.CeasedExistence:
                    value1 = 'N/A'
                value2 = partition2.prop(part_prop)
                print(f"Property '{part_prop}' on partition objects 1: "
                      f"{value1!r}, 2: {value2!r}")
                sleep(1)
        except KeyboardInterrupt:
            print("Keyboard interrupt - leaving loop")
        finally:
            print("Disabling auto-update for partition object 1")
            partition1.disable_auto_update()

    finally:
        print(f"Deleting partition {part.name} ...")
        try:
            part.delete()
        except zhmcclient.Error as exc:
            print(f"Error: Cannot delete partition {part.name} on CPC "
                  f"{cpc.name} for clean up - "
                  f"Please delete it manually: {exc.__class__.__name__}: {exc}")
            sys.exit(1)

finally:
    print("Logging off ...")
    session.logoff()



# Copyright 2022 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
HMC definition for zhmcclient end2end tests.
"""


__all__ = ['HMCDefinition']


class HMCDefinition:
    """
    A single HMC definition.

    An HMC definition contains information needed to use the WS API of a real
    HMC or to setup and use a mocked HMC, some organizational information about
    the HMC, and some information about the CPCs managed by the HMC that are to
    be tested. It can also contain arbitrary additional variables.
    """

    def __init__(self, nickname, description='', contact='', access_via='',
                 mock_file=None, host=None, userid=None,
                 password=None, verify=True, ca_certs=None,
                 cpcs=None, add_vars=None):
        """
        Parameters:

          nickname (string): Nickname of the HMC (DNS name, IP address, or
            host alias).

          description (string): Short description of the HMC.

          contact (string): Name of the technical contact of the HMC.

          access_via (string): Networking preconditions for reaching the HMC.

          mock_file (string): Path name of HMC mock file, or `None`.
            This argument is used to detect whether the HMC is a real or mocked
            HMC: If `None`, it is a real HMC. Otherwise, it is a mocked HMC
            and the path name is stored as provided.

          host (string or list of string): IP address or DNS hostname of the
            real HMC or list of redundant HMCs.
            Required for real HMCs, must not be `None`. Optional for mocked
            HMCs.

          userid (string): Userid (username) for authenticating with the HMC.
            Required, must not be `None`.

          password (string): Password for authenticating with the HMC.
            Required, must not be `None`.

          verify (bool): Verify the HMC certificate as specified in
            `ca_certs`. Optional, defaults to `True`.

          ca_certs (string): Path name of certificate file or certificate
            directory to be used for verifying the HMC certificate, or `None`.
            If `None`, the path name in the 'REQUESTS_CA_BUNDLE' environment
            variable, or the path name in the 'CURL_CA_BUNDLE' environment
            variable, or the certificates in the Mozilla CA Certificate List
            provided by the 'certifi' Python package are used.
            Optional, defaults to `None`.

          cpcs (dict): CPCs managed by the HMC that are to be tested.
            If `None`, no CPCs are tested.

            * key: CPC name.
            * value: dict of expected CPC properties, using underscored
              property names. Used for basic classification of the CPC,
              e.g. 'dpm-mode', 'machine-type', 'machine-model'.

          add_vars (dict): Additional variables. The variable values can have
            arbitrary types.
        """
        assert nickname
        self._nickname = nickname
        self._description = description or ''
        self._contact = contact or ''
        self._access_via = access_via or ''
        self._mock_file = mock_file
        if host is None:
            self._hosts = None
        elif isinstance(host, str):
            self._hosts = [host]
        else:
            self._hosts = list(host)
            assert len(self._hosts) >= 1
        self._userid = userid
        self._password = password
        self._verify = verify
        self._ca_certs = ca_certs
        self._cpcs = cpcs or {}
        self._add_vars = add_vars or {}

    def __repr__(self):
        return (
            "HMCDefinition("
            f"nickname={self.nickname!r}, "
            f"description={self.description!r}, "
            f"contact={self.contact!r}, "
            f"access_via={self.access_via!r}, "
            f"mock_file={self.mock_file!r}, "
            f"host={self.host!r}, "
            f"userid={self.userid!r}, "
            f"password=..., "
            f"verify={self.verify!r}, "
            f"ca_certs={self.ca_certs!r}, "
            f"verify_cert={self.verify_cert!r}, "
            f"cpcs={self.cpcs!r}, "
            f"add_vars={self.add_vars!r})")

    @property
    def nickname(self):
        """
        string: Nickname of the HMC (exists only in this encapsulation class,
          not known by the HMC itself).
        """
        return self._nickname

    @property
    def description(self):
        """
        string: Short description of the HMC.
        """
        return self._description

    @property
    def contact(self):
        """
        string: Name of the technical contact of the HMC.
        """
        return self._contact

    @property
    def access_via(self):
        """
        string: Networking preconditions for reaching the HMC.

        For example, Boundary firewall or VPN connection.
        """
        return self._access_via

    @property
    def mock_file(self):
        """
        string: Path name of HMC mock file, or `None`.

        An HMC mock file defines a mocked HMC based on the zhmcclient_mock
        support.

        This property indicates whether the HMC is a real or mocked HMC:
        If `None`, it is a real HMC, otherwise it is a mocked HMC.
        """
        return self._mock_file

    @property
    def host(self):
        """
        string or list of string: IP address or DNS hostname of the HMC or
        list of redundant HMCs.

        This is a settable property.
        """
        if self._hosts is None:
            host = None
        elif len(self._hosts) == 1:
            host = self._hosts[0]
        else:
            host = self._hosts
        return host

    @host.setter
    def host(self, host):
        """Setter method; for a description see the getter method."""
        # pylint: disable=attribute-defined-outside-init
        if host is None:
            self._hosts = None
        elif isinstance(host, str):
            self._hosts = [host]
        else:
            self._hosts = list(host)
            assert len(self._hosts) >= 1

    @property
    def userid(self):
        """
        string: Userid (username) for authenticating with the HMC.
        """
        return self._userid

    @property
    def password(self):
        """
        string: Password for authenticating with the HMC.
        """
        return self._password

    @property
    def verify(self):
        """
        bool: Verify the HMC certificate as specified in :attr:`ca_certs`.
        """
        return self._verify

    @property
    def ca_certs(self):
        """
        string: Path name of certificate file or certificate directory to be
        used for verifying the HMC certificate, or `None`.

        If `None`, the path name in the 'REQUESTS_CA_BUNDLE' environment
        variable, or the path name in the 'CURL_CA_BUNDLE' environment
        variable, or the certificates in the Mozilla CA Certificate List
        provided by the 'certifi' Python package are used.
        """
        return self._ca_certs

    @property
    def verify_cert(self):
        """
        bool or string: A combination of :attr:`verify` and
        :attr:`ca_certs` for direct use as the `verify_cert` parameter of
        :class:`zhmcclient.Session`.
        """
        if self._ca_certs:
            return self._ca_certs
        return self._verify

    @property
    def cpcs(self):
        """
        dict: CPCs managed by the HMC that are to be tested.

        Each dict item represents one CPC:

        * key (string): CPC name.
        * value (dict): Dict with expected CPC properties (with underscores).
        """
        return self._cpcs

    @property
    def add_vars(self):
        """
        dict: Additional variables for the HMC definition.

        Each dict item represents one variable:

        * key (string): Variable name.
        * value (object): Variable value.
        """
        return self._add_vars



# Copyright 2024 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
End2end tests for virtual storage resources (VSRs) (on CPCs in DPM mode).

These tests do not change any existing VSRs.
"""


import pytest
from requests.packages import urllib3

# pylint: disable=line-too-long,unused-import
from zhmcclient.testutils import hmc_definition, hmc_session  # noqa: F401, E501
from zhmcclient.testutils import dpm_mode_cpcs  # noqa: F401, E501
# pylint: enable=line-too-long,unused-import

from .utils import skip_warn, pick_test_resources, \
    skipif_no_storage_mgmt_feature, runtest_find_list, runtest_get_properties

urllib3.disable_warnings()

# Properties in minimalistic VirtualStorageResource objects
# (e.g. find_by_name())
VSR_MINIMAL_PROPS = ['element-uri', 'name']

# Properties in VirtualStorageResource objects returned by list() without full
# props
VSR_LIST_PROPS = ['element-uri', 'name', 'device-number', 'adapter-port-uri',
                  'partition-uri']

# Properties whose values can change between retrievals of
# VirtualStorageResource objs
VSR_VOLATILE_PROPS = []


def test_vsr_find_list(dpm_mode_cpcs):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test list(), find(), findall().
    """
    if not dpm_mode_cpcs:
        pytest.skip("HMC definition does not include any CPCs in DPM mode")

    for cpc in dpm_mode_cpcs:
        assert cpc.dpm_enabled
        skipif_no_storage_mgmt_feature(cpc)

        session = cpc.manager.session
        hd = session.hmc_definition

        # Pick the VSRs to test with
        grp_vol_tuples = []
        stogrp_list = cpc.list_associated_storage_groups()
        for stogrp in stogrp_list:
            vsr_list = stogrp.virtual_storage_resources.list()
            for vsr in vsr_list:
                grp_vol_tuples.append((stogrp, vsr))
        if not grp_vol_tuples:
            skip_warn("No storage groups with VSRs associated to CPC "
                      f"{cpc.name} managed by HMC {hd.host}")
        grp_vol_tuples = pick_test_resources(grp_vol_tuples)

        for stogrp, vsr in grp_vol_tuples:
            print(f"Testing on CPC {cpc.name} with VSR "
                  f"{vsr.name!r} of storage group {stogrp.name!r}")
            runtest_find_list(
                session, stogrp.virtual_storage_resources, vsr.name, 'name',
                'description',
                VSR_VOLATILE_PROPS, VSR_MINIMAL_PROPS, VSR_LIST_PROPS,
                unique_name=True)


def test_vsr_property(dpm_mode_cpcs):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test property related methods
    """
    if not dpm_mode_cpcs:
        pytest.skip("HMC definition does not include any CPCs in DPM mode")

    for cpc in dpm_mode_cpcs:
        assert cpc.dpm_enabled
        skipif_no_storage_mgmt_feature(cpc)

        session = cpc.manager.session
        hd = session.hmc_definition

        # Pick the VSRs to test with
        grp_vol_tuples = []
        stogrp_list = cpc.list_associated_storage_groups()
        for stogrp in stogrp_list:
            vsr_list = stogrp.virtual_storage_resources.list()
            for vsr in vsr_list:
                grp_vol_tuples.append((stogrp, vsr))
        if not grp_vol_tuples:
            skip_warn("No storage groups with VSRs associated to CPC "
                      f"{cpc.name} managed by HMC {hd.host}")
        grp_vol_tuples = pick_test_resources(grp_vol_tuples)

        for stogrp, vsr in grp_vol_tuples:
            print(f"Testing on CPC {cpc.name} with VSR "
                  f"{vsr.name!r} of storage group {stogrp.name!r}")

            # Select a property that is not returned by list()
            non_list_prop = 'description'

            runtest_get_properties(vsr.manager, non_list_prop)



#!/usr/bin/env python
# Copyright 2017,2022 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example that determines the crypto domains that are free on all crypto adapters
of a CPC in DPM mode.
"""

import sys
import requests.packages.urllib3

import zhmcclient
from zhmcclient.testutils import hmc_definitions

requests.packages.urllib3.disable_warnings()

# Get HMC info from HMC inventory and vault files
hmc_def = hmc_definitions()[0]
nickname = hmc_def.nickname
host = hmc_def.host
userid = hmc_def.userid
password = hmc_def.password
verify_cert = hmc_def.verify_cert

print(__doc__)

print(f"Using HMC {nickname} at {host} with userid {userid} ...")

print("Creating a session with the HMC ...")
try:
    session = zhmcclient.Session(
        host, userid, password, verify_cert=verify_cert)
except zhmcclient.Error as exc:
    print(f"Error: Cannot establish session with HMC {host}: "
          f"{exc.__class__.__name__}: {exc}")
    sys.exit(1)

try:
    client = zhmcclient.Client(session)

    print("Finding a CPC in DPM mode ...")
    cpcs = client.cpcs.list(filter_args={'dpm-enabled': True})
    if not cpcs:
        print(f"Error: HMC at {host} does not manage any CPCs in DPM mode")
        sys.exit(1)
    cpc = cpcs[0]
    print(f"Using CPC {cpc.name}")

    print(f"Finding all crypto adapters of CPC {cpc.name} ...")
    crypto_adapters = cpc.adapters.findall(type='crypto')
    crypto_adapter_names = [ca.name for ca in crypto_adapters]
    print("Found crypto adapters:")
    for ca in crypto_adapters:
        print(f"  {ca.name} (type: {ca.get_property('crypto-type')})")

    print("Determining crypto domains that are free on all crypto adapters of "
          f"CPC {cpc.name} ...")
    free_domains = cpc.get_free_crypto_domains(crypto_adapters)
    # print(f"Free domains (as list): {free_domains}")

    # Convert this list of numbers into better readable number ranges:
    ranges = []
    range_start = -1
    last_d = -1
    for d in sorted(free_domains):
        if range_start == -1:
            range_start = d
        elif d == last_d + 1:
            pass
        else:
            if range_start == last_d:
                ranges.append(f"{last_d}")
            else:
                ranges.append(f"{range_start}-{last_d}")
            range_start = d
        last_d = d
        continue
    if range_start != -1:  # Process the last range, if any
        if range_start == last_d:
            ranges.append(f"{last_d}")
        else:
            ranges.append(f"{range_start}-{last_d}")
    free_domains_str = ', '.join(ranges)
    print(f"Free domains (as ranges): {free_domains_str}")

finally:
    print("Logging off ...")
    session.logoff()
